# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
import pytest


@pytest.fixture(autouse=True)
def reset_helpers_namespace(request):
    try:
        yield
    finally:
        plugin = request.config.pluginmanager.get_plugin("helpers-namespace")
        plugin._registry.clear()


def test_namespace(pytester):
    pytester.makeconftest(
        """
        import pytest

        @pytest.helpers.register
        def foo(bar):
            return bar
        """
    )

    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            assert pytest.helpers.foo(True) is True
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "test_namespace.py PASSED",
        ]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret == 0


def test_nested_namespace(pytester):
    pytester.makeconftest(
        """
        import pytest

        @pytest.helpers.foo.bar.register
        def foo(bar):
            return bar
        """
    )

    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            assert pytest.helpers.foo.bar.foo(True) is True
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "test_nested_namespace.py PASSED",
        ]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret == 0


def test_unregistered_namespace(pytester):
    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            with pytest.raises(RuntimeError) as exc:
                assert pytest.helpers.foo(True) is True
            strexc = str(exc)
            assert 'The helper being called was not registered' in strexc
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "test_unregistered_namespace.py PASSED",
        ]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret == 0


def test_namespace_override(pytester):
    pytester.makeconftest(
        """
        import pytest

        @pytest.helpers.foo.register
        def bar(bar):
            return bar

        @pytest.helpers.register
        def foo(bar):
            return bar
        """
    )
    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            with pytest.raises(RuntimeError) as exc:
                assert pytest.helpers.foo(True) is True
            assert 'The helper being called was not registered' in str(exc)
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stderr.fnmatch_lines(
        ["*RuntimeError: A helper function is already registered under the name: foo"]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret != 0


def test_helper_override(pytester):
    pytester.makeconftest(
        """
        import pytest

        @pytest.helpers.register
        def foo(bar):
            return bar

        @pytest.helpers.register
        def foo(bar):
            return bar
        """
    )
    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            with pytest.raises(RuntimeError) as exc:
                assert pytest.helpers.foo(True) is True
            assert 'The helper being called was not registered' in str(exc)
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stderr.fnmatch_lines(
        ["*RuntimeError: A helper function is already registered under the name: foo"]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret != 0


def test_helper_as_regular_function(pytester):
    pytester.makepyfile(
        """
        import pytest

        @pytest.helpers.register
        def foo2():
            return 'bar'

        def test_helpers():
            assert pytest.helpers.foo2() == 'bar'
            assert foo2() == 'bar'
        """
    )

    result = pytester.runpytest("-svv", "--log-cli-level=debug")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["test_helper_as_regular_function.py::test_helpers PASSED"])

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret == 0


def test_helper_with_custom_name(pytester):
    pytester.makepyfile(
        """
        import pytest

        @pytest.helpers.register('jump')
        def foo():
            return 'bar'

        def test_helpers():
            assert pytest.helpers.jump() == 'bar'
            assert foo() == 'bar'
            print('PASSED')
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "test_helper_with_custom_name.py PASSED",
        ]
    )

    # make sure that that we get a '0' exit code for the test suite
    assert result.ret == 0


def test_helper_contains_method(pytester):
    pytester.makeconftest(
        """
        import pytest

        assert "bar" not in pytest.helpers

        @pytest.helpers.register
        def bar():
            return True
        """
    )
    pytester.makepyfile(
        """
        import pytest

        def test_it():
            assert "bar" in pytest.helpers
            assert pytest.helpers.bar() is True
        """
    )

    result = pytester.runpytest("-vv")
    result.assert_outcomes(passed=1)


def test_call_register_on_helper_function(pytester):
    pytester.makeconftest(
        """
        import pytest

        @pytest.helpers.register
        def foo(bar):
            return bar

        @pytest.helpers.foo.register
        def blah(blah):
            return bar
        """
    )
    pytester.makepyfile(
        """
        import pytest

        def test_helpers():
            with pytest.raises(RuntimeError) as exc:
                assert pytest.helpers.foo(True) is True
    """
    )

    result = pytester.runpytest("-s")

    # fnmatch_lines does an assertion internally
    result.stderr.fnmatch_lines(
        [
            "*RuntimeError: Helper functions cannot be used to register new helper functions. "
            "Register and use a namespace for that.*",
        ]
    )
    # make sure that that we get a '0' exit code for the test suite
    assert result.ret != 0



#!/usr/bin/env python
# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
# pylint: skip-file
import pathlib

REPO_ROOT = pathlib.Path(__name__).resolve().parent
PYLINT_SPELLING_WORDS = REPO_ROOT / ".pylint-spelling-words"


def sort():
    in_contents = PYLINT_SPELLING_WORDS.read_text()
    out_contents = ""
    out_contents += "\n".join(sorted({line.lower() for line in in_contents.splitlines()}))
    out_contents += "\n"
    if in_contents != out_contents:
        PYLINT_SPELLING_WORDS.write_text(out_contents)


if __name__ == "__main__":
    sort()



#!/usr/bin/env python3
# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
# pylint: disable=invalid-name,missing-module-docstring,missing-function-docstring
import argparse
import pathlib
import re
import sys
from datetime import datetime

CODE_ROOT = pathlib.Path(__file__).resolve().parent.parent
SPDX_HEADER = "# SPDX-License-Identifier: Apache-2.0"
COPYRIGHT_HEADER = "# Copyright {year} VMware, Inc."
COPYRIGHT_REGEX = re.compile(
    r"# Copyright (?:(?P<start_year>[0-9]{4})(?:-(?P<cur_year>[0-9]{4}))?) VMware, Inc\."
)
SPDX_REGEX = re.compile(r"# SPDX-License-Identifier:.*")


def check_copyright(files):
    for file in files:
        contents = file.read_text()
        if not contents.strip():
            # Don't add headers to empty files
            continue
        original_contents = contents
        try:
            if not COPYRIGHT_REGEX.search(contents):
                contents = inject_copyright_header(contents)
                if contents != original_contents:
                    print(f"Added the copyright header to {file}")
            else:
                contents = update_copyright_header(contents)
                if contents != original_contents:
                    print(f"Updated the copyright header on {file}")
            if not SPDX_REGEX.search(contents):
                contents = inject_spdx_header(contents)
                if contents != original_contents:
                    print(f"Added the SPDX header to {file}")
        finally:
            if original_contents != contents:
                file.write_text(contents)


def inject_copyright_header(contents):
    lines = contents.splitlines()
    shebang_found = False
    for idx, line in enumerate(lines[:]):
        if idx == 0 and line.startswith("#!"):
            shebang_found = True
            continue
        if shebang_found and line.strip():
            shebang_found = False
            lines.insert(idx, "")
            idx += 1
        lines.insert(idx, COPYRIGHT_HEADER.format(year=datetime.today().year))
        break
    return "\n".join(lines)


def update_copyright_header(contents):
    lines = contents.splitlines()
    for idx, line in enumerate(lines[:]):
        match = COPYRIGHT_REGEX.match(line)
        if match:
            this_year = str(datetime.today().year)
            initial_year = match.group("start_year").strip()
            if initial_year == this_year:
                return contents
            lines[idx] = COPYRIGHT_HEADER.format(year=f"{initial_year}-{this_year}")
            break
    return "\n".join(lines)


def inject_spdx_header(contents):
    lines = contents.splitlines()
    for idx, line in enumerate(lines[:]):
        if COPYRIGHT_REGEX.match(line):
            lines.insert(idx + 1, SPDX_HEADER)
            next_line = lines[idx + 2].strip()
            if next_line and not next_line.startswith('"""'):
                # If the next line is not empty, insert an empty comment
                lines.insert(idx + 2, "#")
            break
    return "\n".join(lines)


def main(argv):
    parser = argparse.ArgumentParser(prog=__name__)
    parser.add_argument("files", nargs="+", type=pathlib.Path)

    options = parser.parse_args(argv)
    return check_copyright(options.files)


if __name__ == "__main__":
    sys.exit(main(sys.argv))
